//Doom 3 styled flashlight, requires DarkDoomZ edit from Xim-Doom3-world.pk3
//Weapon can not be switched until flashlight is turned off
//Sprites may clash with weapon mods that don't use the doomguy hands
//Sprite angle can be adjusted with DDZ flashlight position setting
class XD3_Flashlight : Weapon
{
	Default
	{
		AttackSound "weapons/flitehit";
		Weapon.SelectionOrder 4600;
		Weapon.Kickback 100;
		Obituary "%o was bashed by %k's flashlight.";
		Tag "Flashlight";
		+WEAPON.MELEEWEAPON
		+WEAPON.NOAUTOSWITCHTO
	}
	States
	{
	Ready:
		TNT1 A 0
		{
			if(CountInv("FlashlightActiveTrigger") == 0)//checks for active trigger
			{
				A_StartSound("DDZ_Flashlight_On", chan_item);
				Inventory Flashlight = FindInventory("DarkDoomZ_Flashlight");//records flashlight item
				//UseInventory(Flashlight);//triggers recorded flashlight item
				A_GiveInventory("FlashlightActiveTrigger", 1);//gives active trigger
			}
		}
	SetReady://Sprite set based on flashlight position setting
		TNT1 A 0 A_JumpIf(GetCvar("ddz_fl_pos") == 1,"L_Ready");
		TNT1 A 0 A_JumpIf(GetCvar("ddz_fl_pos") == 2,"R_Ready");
		TNT1 A 0 A_JumpIf(GetCvar("ddz_fl_pos") == 3,"C_Ready");
	H_Ready://handheld
		FLIT A 1 A_WeaponReady(WRF_NOSWITCH); 
		Loop;
	R_Ready://right
		FLIT A 1 A_WeaponReady(WRF_NOSWITCH|WRF_NOBOB);
		Loop;		
	C_Ready://centre
		FLIT B 1 A_WeaponReady(WRF_NOSWITCH|WRF_NOBOB);
		Loop;
	L_Ready://left
		FLIT C 1 A_WeaponReady(WRF_NOSWITCH|WRF_NOBOB);
		Loop;		
	Deselect:
		TNT1 A 0 A_StartSound("DDZ_Flashlight_Off", chan_item);
		TNT1 A 0 A_JumpIf(GetCvar("ddz_fl_pos") == 1,"L_Deselect");
		TNT1 A 0 A_JumpIf(GetCvar("ddz_fl_pos") == 3,"C_Deselect");
	R_Deselect://handheld/right
		FLIT A 1 A_Lower(22);
		Loop;
	C_Deselect://centre
		FLIT B 1 A_Lower(22);
		Loop;
	L_Deselect://left
		FLIT C 1 A_Lower(22);
		Loop;		
	Select:
		TNT1 A 0 A_GiveInventory("XD3FlashlightInHand", 1);
		TNT1 A 0 A_JumpIf(GetCvar("ddz_fl_pos") == 1,"L_Select");
		TNT1 A 0 A_JumpIf(GetCvar("ddz_fl_pos") == 3,"C_Select");
	R_Select://handheld/right
		FLIT A 1 A_Raise(22);
		Loop;
	C_Select://centre
		FLIT B 1 A_Raise(22);
		Loop;
	L_Select://left
		FLIT C 1 A_Raise(22);
		Loop;		
	Fire:
		D3FG A 0 A_GiveInventory("FlashlightCooldown");//gives cooldown to turn off flashlight, but still keep it 'active'
		D3FG A 3 A_StartSound("weapons/fliteswing", chan_weapon);
		D3FG BCD 3;
		D3FG D 3 A_CustomPunch(4 * random(2, 5), false, CPF_NOTURN, "FlashlightPuff");
		D3FG EF 3;
		D3FG F 0 
		{
			//removes cooldown and reactivates the flashlight
			A_TakeInventory("FlashlightCooldown", 0);
			Inventory Flashlight = FindInventory("DarkDoomZ_Flashlight");
			UseInventory(Flashlight);
		}
		D3FG F 0 A_ReFire;
		Goto SetReady;
	}
}

class FlashlightPuff : BulletPuff
{
	Default
	{
		+NOEXTREMEDEATH
		DamageType "Melee";
	}
	States
	{
		Crash:
			TNT1 A 0 A_StartSound("weapons/flitehitwall");
			TNT1 A 0;
			Goto Spawn;
		XDeath:
			TNT1 A 0 A_StartSound("weapons/flitehit");
			TNT1 A 0;
			Goto Spawn;
	}
}

//Flashlight Battery
Class FlashlightBattery : Ammo
{
	Default
	{
		Inventory.MaxAmount 100;
		Inventory.Icon "FLSHB0";
		-INVENTORY.INVBAR;
		Tag "Flashlight";
	}
}
Class FlashlightCooldown : Powerup
{
	Default
	{
		Powerup.Duration 101;//duration in tics for flashlight cooldown
	}
}

Class FlashlightActiveTrigger : Inventory //for preventing flashlight from double activating between levels
{
	Default
	{
		inventory.maxamount 1;
	} 
}

Class XD3FlashlightInHand : Powerup //for the D3 hud
{
	Default
	{
		Powerup.Duration 20;
	}
}

//Flashlight Pickup, gives flashlight and full battery
class DarkDoomZ_Flashlight_XD3Pickup : CustomInventory
{
	Default
	{
		Inventory.PickupMessage "You got the flashlight!";
		Inventory.PickupSound "flashlight/pickup";
		Mass 10;
	}
	States
	{
		Spawn:
			FLSH A -1;
			stop;
		Pickup:
			TNT1 A 0 A_GiveInventory("DarkDoomZ_Flashlight", 1);
			TNT1 A 0 A_GiveInventory("FlashlightBattery", 100);
			Stop;
	}
}