// darker_events.zs

// developed with "Standard" sector lighting

// https://github.com/zdoom-docs/stable/blob/master/api/level/Sector.md
// https://github.com/zdoom-docs/stable/blob/master/api/level/Side.md

class darker_EventHandler : EventHandler
{
	struct flatcolor
	{
		textureid texid;
		color texcolor;
		int desat;
		color fade;
		int dens;
		int glow_floor;
		int glow_ceiling;
	}
	struct sidecolor
	{
		textureid texid;
		color texcolor;
		int desat;
		int light;
	}

	flatcolor FlatColors[50];
	sidecolor SideColors[50];
	int flatCounter;
	int sideCounter;

	void AddFlatColor(string tname, string tcolor, int desat = 0, string fade = "", int dens = 0, int glow_floor = 0, int glow_ceiling = 0)
	{
		Texman texture;
		textureid id = texture.CheckForTexture(tname, Texman.TYPE_ANY, Texman.TRYANY | Texman.OVERRIDABLE);

		// console.printf("Found %d for %s", id, tname);

		FlatColors[flatCounter].texid = id;
		FlatColors[flatCounter].texcolor = tcolor;
		FlatColors[flatCounter].desat = desat;
		FlatColors[flatCounter].fade = fade;
		FlatColors[flatCounter].dens = dens;
		FlatColors[flatCounter].glow_floor = glow_floor;
		FlatColors[flatCounter].glow_ceiling = glow_ceiling;

		flatCounter++;
	}
	void AddSideColor(string tname, string tcolor, int desat = 0, int light = 0)
	{
		Texman texture;
		textureid id = texture.CheckForTexture(tname, Texman.TYPE_ANY, Texman.TRYANY | Texman.OVERRIDABLE);

		SideColors[sideCounter].texid = id;
		SideColors[sideCounter].texcolor = tcolor;
		FlatColors[sideCounter].desat = desat;
		SideColors[sideCounter].light = light;
		
		sideCounter++;
	}
	void LoadColors()
	{
		flatCounter = 0;
		sideCounter = 0;
		
		// floor and ceiling sector colors
		AddFlatColor("TLITE6_5", "#ff0000");
		//AddFlatColor("BLOOD1", "#ff0000");
		//AddFlatColor("BLOOD2", "#ff0000");
		//AddFlatColor("BLOOD3", "#ff0000");
		AddFlatColor("TLITE6_1", "#ffff00");
		AddFlatColor("TLITE6_4", "#ffff00");
		AddFlatColor("TLITE6_6", "#ffff00");
		AddFlatColor("NUKAGE1", "#00ff00", 0, "#00ff00", 1, 32);
		AddFlatColor("NUKAGE2", "#00ff00", 0, "#00ff00", 1, 32);
		AddFlatColor("NUKAGE3", "#00ff00", 0, "#00ff00", 1, 32);
		AddFlatColor("LAVA1", "#ff8c00");
		AddFlatColor("LAVA2", "#ff8c00");
		AddFlatColor("LAVA3", "#ff8c00");
		AddFlatColor("LAVA4", "#ff8c00");
		
		AddFlatColor("BROCK01", "#ff8c00");
		AddFlatColor("BROCK02", "#ff8c00");
		AddFlatColor("BROCK03", "#ff8c00");
		AddFlatColor("BROCK04", "#ff8c00");
		
		AddFlatColor("LROCK01", "#ff8c00");
		AddFlatColor("LROCK02", "#ff8c00");
		AddFlatColor("LROCK03", "#ff8c00");
		AddFlatColor("LROCK04", "#ff8c00");

		// sidedef sector colors
		AddSideColor("LITERED", "#ff0000", 0, 255);
		AddSideColor("LITE2", "#ffff00", 0, 255);
		AddSideColor("LITE3", "#ffff00", 0, 255);
		AddSideColor("LITE4", "#ffff00", 0, 255);
		AddSideColor("LITE5", "#ffff00", 0, 255);
		AddSideColor("LITE96", "#ffff00", 0, 255);
		AddSideColor("LITEBLU1", "#0000ff", 0, 255);
		AddSideColor("LITEBLU4", "#0000ff", 0, 255);
		AddSideColor("LITESTON", "#ffff00", 0, 255);

		AddSideColor("COMPSTA1", "#b3c8ea", 0, 192);
		AddSideColor("COMPSTA2", "#b3c8ea", 0, 192);

		AddSideColor("DOORRED", "#ff0000", 0, 255);
		AddSideColor("DOORRED2", "#ff0000");
		AddSideColor("DOORYEL", "#ffff00");
		AddSideColor("DOORBLU", "#0000ff");
		AddSideColor("DOORBLU2", "#0000ff");
		
		AddSideColor("SFALL1", "#00ff00");
		AddSideColor("SFALL2", "#00ff00");
		AddSideColor("SFALL3", "#00ff00");
		AddSideColor("SFALL4", "#00ff00");
		
		AddSideColor("GFALL1", "#ffff00");
		AddSideColor("GFALL2", "#ffff00");
		AddSideColor("GFALL3", "#ffff00");
		AddSideColor("GFALL4", "#ffff00");
	}
	int lightstep(int lightlevel)
	{
		int dk_low =  Cvar.FindCvar("dk_low").GetInt();
		int dk_med =  Cvar.FindCvar("dk_med").GetInt();
		int dk_hi =  Cvar.FindCvar("dk_hi").GetInt();
		int dk_strength =  Cvar.FindCvar("dk_strength").GetInt();
		return lightlevel - lightlevel < dk_low ? dk_strength * 4 : lightlevel < dk_med ? dk_strength * 2 : lightlevel < dk_hi ? dk_strength : 0;
	}
	int newlightlevel(int minadjacent, int lightlevel)
	{
		double dk_step = Cvar.FindCvar("dk_step").GetFloat();
		return minadjacent < lightlevel ? lightlevel - (lightstep(lightlevel) * dk_step) : lightlevel - lightstep(lightlevel);
	}

	override
	void WorldLoaded(WorldEvent e)
	{
		int FLOOR = 0;
		int CEILING = 1;
		int INNER = 0;
		int dk_hi =  Cvar.FindCvar("dk_hi").GetInt();

		TexMan texture;
		Array<int> darker;

		LoadColors();

		// fill darker array
		for(int i = 0; i < Level.Sectors.Size(); i++)
		{
			int lightlevel = Level.Sectors[i].lightlevel;
			int minadjacent = Level.Sectors[i].FindMinSurroundingLight(dk_hi);
			int newlightlevel = newlightlevel(minadjacent, lightlevel);
			darker.Push(newlightlevel);
		}
		// apply light levels and colors
		for(int i = 0; i < Level.Sectors.Size(); i++)
		{
			//Level.Sectors[i].SetLightLevel(darker[i]);
			//Level.Sectors[i].SetPlaneLight(CEILING, lightstep(darker[i]) * -1);

			if (Cvar.FindCVar("dk_color").GetBool())
			{
				// floor and ceiling
				textureid floorid = Level.Sectors[i].GetTexture(FLOOR);
				textureid ceilingid = Level.Sectors[i].GetTexture(CEILING);
				
				//textureid skyid = texture.CheckForTexture("F_SKY1", Texman.TYPE_ANY, Texman.TRYANY | Texman.OVERRIDABLE);//from relighting.pk3

				bool sides = true;
				for (int ii = 0; ii < flatCounter; ii++)
				{
					if (floorid == FlatColors[ii].texid || ceilingid == FlatColors[ii].texid)
					{
						Level.Sectors[i].SetColor(FlatColors[ii].texcolor, FlatColors[ii].desat);

						Level.Sectors[i].SetFade(FlatColors[ii].fade);
						Level.Sectors[i].SetFogDensity(FlatColors[ii].dens);
						Level.Sectors[i].SetGlowColor(FLOOR, FlatColors[ii].fade);
						Level.Sectors[i].SetGlowHeight(FLOOR, FlatColors[ii].glow_floor);
						Level.Sectors[i].SetGlowColor(CEILING, FlatColors[ii].fade);
						Level.Sectors[i].SetGlowHeight(CEILING, FlatColors[ii].glow_ceiling);

						sides = false;
						break;
					}
					// console.printf("None for floor %s and ceiling %s", texture.GetName(floorid), texture.GetName(floorid));
				}
				
				//for(int i = 0; i < Level.Sectors.Size(); i++)//from relighting.pk3
				//{
				//	Sector sec = Level.Sectors[i];
				//	SecPlane floorplane = sec.FloorPlane;
				//	SecPlane ceilingplane = sec.CeilingPlane;

				//	string cname = "LightSensor";
				//	textureid ceilingid = sec.GetTexture(CEILING);
				//	if (ceilingid == skyid)
				//	{
				//		sec.SetLightLevel(224);
				//		cname = "SkyLight";
				//	}
				//}
				
				// sidedefs
				if (sides)
				{
					for (int j = 0; j < Level.Sectors[i].Lines.Size(); j++)
					{
						Side sidedef = Level.Sectors[i].Lines[j].Sidedef[INNER];
						textureid topid = sidedef.GetTexture(sidedef.Top);
						textureid midid = sidedef.GetTexture(sidedef.Mid);
						textureid botid = sidedef.GetTexture(sidedef.Bottom);
						
						for (int ii = 0; ii < sideCounter; ii++)
						{
							if (topid == SideColors[ii].texid || midid == SideColors[ii].texid || botid == SideColors[ii].texid)
							{
								Level.Sectors[i].SetColor(SideColors[ii].texcolor, SideColors[ii].desat);
								if (SideColors[ii].light > 0)
								{
									Level.Sectors[i].Lines[j].Sidedef[INNER].flags = Level.Sectors[i].Lines[j].Sidedef[INNER].flags | Side.WALLF_ABSLIGHTING | Side.WALLF_SMOOTHLIGHTING;
									Level.Sectors[i].Lines[j].Sidedef[INNER].light = SideColors[ii].light;
								}
								break;
							}
							// console.printf("None for top %s mid %s bot %s", texture.GetName(topid), texture.GetName(midid), texture.GetName(botid));
						}
					}
				}
			}
		}
	}
}


class darker_OptionMenu : OptionMenu {
	override void Init(Menu parent, OptionMenuDescriptor desc) {
		super.Init(parent, desc);
		DontDim = true;
		DontBlur = true;
	}
}